import vlc
import requests
import click


def player(stream_url):
    """
    Takes the stream url and plays the audio in background process while listening for input controls.
    :param stream_url:
    :return:
    """
    valid_media_extensions = [
        "audio/mpeg",
        "audio/aac",
        "application/ogg",
        "audio/x-m4a",
        "audio/3gpp",
        "audio/3gpp2",
        "audio/opus",
        "audio/vorbis",
    ]
    resp_content_type = requests.get(stream_url, stream=True, timeout=50).headers.get(
        "Content-Type"
    )

    if resp_content_type in valid_media_extensions:
        instance_vlc = vlc.Instance("--quiet")
        # Suppresses prefetch stream errors to stdout.

        media_player = instance_vlc.media_player_new()
        media_url_vlc_obj = instance_vlc.media_new(stream_url)
        media_player.set_media(media_url_vlc_obj)

        media_player.play()
        print("Radio stream has started ... ((d[-_-]b))\n")
        print("NOTE: If you can't hear anything, try increasing volume ...\n")

        media_control_input = None
        while not media_control_input == "q":
            click.echo(
                "Enter q to quit, p to play/pause, m to mute/unmute, +/- to change "
                "volume. \ndev-radio>",
                nl=False,
            )
            media_control_input = click.getchar().lower()
            click.echo()
            volume_step = 10
            # Volume is decreased/increased in steps.

            if media_control_input == "p":
                media_player.pause()
                print("Play/Pause (^_^)\n")

            elif media_control_input == "q":
                print("Aww, man you killed the radio. (T_T)\n")
                media_player.stop()
                break

            elif media_control_input == "m":
                print("Mute/Unmute (~_~)\n")
                media_player.audio_toggle_mute()

            elif media_control_input == "+":
                current_volume = media_player.audio_get_volume()
                current_volume += volume_step
                max_volume = 151
                if current_volume <= max_volume:
                    media_player.audio_set_volume(current_volume)
                    print(f"+vol {current_volume}\n")
                else:
                    print(
                        "Max volume reached, you will bleed through ears, if you go any further. (o_O)\n"
                    )

            elif media_control_input == "-":
                current_volume = media_player.audio_get_volume()
                current_volume -= volume_step
                min_volume = 0
                if current_volume >= min_volume:
                    media_player.audio_set_volume(current_volume)
                    print(f"-vol {current_volume}\n")
                else:
                    print("Min volume reached (-_-)zzz\n")

            else:
                print("Incorrect input choice, try again ... (?_?)\n")

    else:
        print("Invalid url:: url maybe dead ...")



import click
from src.radio_stations import (
    cli_radio_station_list,
    cli_radio_play,
    cli_add_station,
    cli_del_station,
    cli_reset_stations,
    cli_radio_check,
    cli_ren_station,
)
from src.podcasts import cli_podcast_list, cli_print_episodes, cli_podcast_play

"""Entry point for dev-radio."""


class AliasedGroup(click.Group):
    """
    A class for creating aliases for commands.
    """

    def get_command(self, ctx, cmd_name):
        try:
            cmd_name = ALIASES[cmd_name].name
        except KeyError:
            pass
        return super().get_command(ctx, cmd_name)


@click.group(cls=AliasedGroup)
def dradio():
    """
    dev-radio is simple command line tool to listen to python podcast and radio streams.
    Use --help with Commands to get their help.
    """
    pass


radio_play_help = """Pass the radio station name, to start playing radio.

\b
Example: dradio radio --play <radio_station_name>
"""
radio_add_help = """Add new radio station by passing radio_name and radio_url, resp

\b
Example: dradio radio --add-station <radio_station_name> <radio_station_url>
"""
radio_del_help = """Deletes the radio station of the said name.

\b
Example: dradio radio --del-station <radio_station_name>
"""
radio_ren_help = """Renames the radio station. This just performs delete and add station.

\b
Example: dradio radio --rename-station <old_station_name> <new_station_name>
"""


@dradio.command(help="ALIAS: rad, Plays radio streams.")
@click.option(
    "--list", "-L", "list_", is_flag=True, help="Lists all the radio stations."
)
@click.option("--play", "-P", type=str, help=radio_play_help)
@click.option(
    "--add-station", "-A", "add_station", nargs=2, type=str, help=radio_add_help
)
@click.option("--del-station", "-D", "del_station", type=str, help=radio_del_help)
@click.option(
    "--rename-station", "-R", "ren_station", nargs=2, type=str, help=radio_ren_help
)
@click.option(
    "--reset",
    is_flag=True,
    help="Resets all the radio stations to default state, may come in handy if you delete some stations.",
)
@click.option(
    "--check", "-C", is_flag=True, help="Checks which radio urls are working."
)
def radio(list_, play, add_station, del_station, ren_station, reset, check):
    if list_:
        cli_radio_station_list()
    elif play:
        cli_radio_play(play)
    elif add_station:
        print("Wait for a few seconds, checking if the url is online ...")
        cli_add_station(add_station[0], add_station[1])
    elif del_station:
        if click.confirm(f"Do you want to delete {del_station} station?", abort=True):
            cli_del_station(del_station)
    elif ren_station:
        cli_ren_station(ren_station[0], ren_station[1])
    elif reset:
        if click.confirm("Do you want to reset all radio stations?", abort=True):
            cli_reset_stations()
    elif check:
        cli_radio_check()


podcast_play_help = """Pass the podcast name and episode_id (NOT the episode_number) to play the episode,
resp. pass <podcast_episode_id> as 0 for latest episode.

\b
Example: dradio podcast <podcast_name> <podcast_episode_id>
"""


@dradio.command(help="ALIAS: pod, Plays podcasts.")
@click.option("--list", "-L", "pod_list_", is_flag=True, help="Lists all the podcasts.")
@click.option(
    "--all-eps",
    "-A",
    "eps",
    type=str,
    help="Shows all the episodes of podcast <podcast_name>",
)
@click.option(
    "--play", "-P", type=(str, int), default=(None, None), help=podcast_play_help
)
def podcast(pod_list_, eps, play):
    if pod_list_:
        cli_podcast_list()
    elif eps:
        cli_print_episodes(eps)
    elif play:
        if None not in play:
            cli_podcast_play(podcast_name=play[0], episode_id=play[1])


# Alias dictionary for commands.
ALIASES = {"rad": radio, "pod": podcast}

if __name__ == "__main__":
    dradio()



from src.player import player
import json
from pathlib import Path
from prettytable import PrettyTable, ALL
import requests
from requests.exceptions import MissingSchema, ConnectionError


# noinspection SpellCheckingInspection
radio_urls = {
    "jpop": "https://listen.moe/stream",
    "anime": "https://s3.radio.co/sff133d65b/listen",
    "jhits": "https://igor.torontocast.com/JapanHits",
    "jrock": "https://kathy.torontocast.com:3340/;",
    "jpop0": "https://kathy.torontocast.com:3560/;",
    "anime0": "https://stream.laut.fm/animefm",
    "kpop": "https://listen.moe/kpop/stream",
    "eurobeat": "https://stream.laut.fm/eurobeat",
    "vocaloid": "http://curiosity.shoutca.st:8019/stream",
    "synthwave": "http://air.radiorecord.ru:805/synth_320",
    "synthwave0": "https://stream.nightride.fm/nightride.m4a",
    "synthwave1": "https://ecast.myautodj.com/public1channel",
    "dance": "https://www.ophanim.net:8444/s/9780",
    "dance0": "https://stream.laut.fm/dance",
    "dubstep": "https://stream.24dubstep.pl/radio/8000/mp3_best",
    "dubstep0": "https://radio.maddubz.net/radio/8000/dubstep.mp3",
    "dubstep1": "https://ice6.somafm.com/dubstep-128-mp3",
    "country": "https://pureplay.cdnstream1.com/6029_128.mp3",
    "amazing_80s": "https://stream.amazingradios.com/80s",
    "heavy_metal": "https://stream.laut.fm/metal",
    "lo_fi": "https://laut.fm/lofi",
    "chill": "https://stream.laut.fm/loungetunes",
    "hip_hop": "https://stream.laut.fm/1000hiphop",
}


def read_stations():
    """
    reads the radio station json file and returns dict.

    :return: dict{'radio_station': 'url'}
    """
    file_path = Path(__file__).parent / "radio_stations.json"
    with open(file_path, "r") as infile:
        old_radio_data = json.load(infile)
    return old_radio_data


def cli_radio_station_list():
    """
    CLI facing function that pretty prints all the radio stations.

    :return:
    """
    pretty_table = PrettyTable()
    pretty_table.hrules = ALL
    pretty_table.field_names = ["id", "Station"]
    station_list = list(read_stations().keys())
    station_list = [
        (index, station) for index, station in enumerate(station_list, start=0)
    ]
    pretty_table.add_rows(station_list)
    print(pretty_table)


def station_config(operation, new_station_key, new_station_value=None):
    """
    Adds/Deletes radio station to radio_stations.json file.

    :param operation: 'a' adds new station with new station key and value, 'd' deletes station with new_station_key
    :param new_station_key: radio station name.
    :param new_station_value: radio station url.
    :return:
    """
    old_radio_data = read_stations()
    if operation.lower() == "a":
        old_radio_data.update({new_station_key: new_station_value})
        print(f"Added new station {new_station_key} with url :: {new_station_value}")
    elif operation.lower() == "d":
        try:
            popped_station = old_radio_data.pop(new_station_key)
        except KeyError:
            print(
                f"Station named {new_station_key} that you are trying to delete doesn't exist."
            )
        else:
            print(
                f"Station named {new_station_key} and url {popped_station} was deleted."
            )

    else:
        print("Incorrect Configuration Option.")

    file_path = Path(__file__).parent / "radio_stations.json"
    with open(file_path, "w") as outfile:
        json.dump(old_radio_data, outfile, indent=4)


def cli_radio_play(station_name):
    """
    CLI facing function that plays radio named station_name.

    :param station_name: radio station name.
    :return:
    """
    station_url = read_stations().get(station_name)
    if station_url:
        print(f"Station :: {station_name}")
        player(station_url)
    else:
        print("Station name is incorrect, try again.")


def cli_add_station(station_name, station_url):
    """
    CLI facing wrapper function around station_config() that adds new radio station to file.

    :param station_name: radio station name.
    :param station_url: radio station url.
    :return:
    """
    try:
        resp = requests.get(station_url, stream=True, timeout=50)
    except MissingSchema:
        print("Stream url is incomplete ...")
    except ConnectionError:
        print("Either you are offline or the site is ...")
    except requests.exceptions.Timeout:
        print("Request Time out exception when trying to access radio station ...")
    except requests.exceptions.RequestException as e:
        print(e)
        print("There seems to be some problem with Url ...")
    else:
        station_config(
            operation="a", new_station_key=station_name, new_station_value=station_url
        )


def cli_del_station(station_name):
    """
    CLI facing wrapper function for station_config() that deletes radio stations from the file.

    :param station_name: radio station name.
    :return:
    """
    station_config(operation="d", new_station_key=station_name)


def cli_ren_station(old_station_name, new_station_name):
    """
    CLI facing function that renames the radio stations.
    This is just performs delete and add station.

    :param old_station_name: old radio station name.
    :param new_station_name: new radio station name.
    :return:
    """
    radio_stations = read_stations()
    try:
        new_station_url = radio_stations.pop(old_station_name)
    except KeyError:
        print(f"Station name {old_station_name} doesn't exist ...")
    else:
        cli_del_station(old_station_name)
        cli_add_station(new_station_name, new_station_url)
        print(f"{old_station_name} Renamed --> {new_station_name}")


def cli_reset_stations():
    """
    CLI facing function that resets the radio stations stored in radio_stations.json to default ones.

    :return:
    """
    file_path = Path(__file__).parent / "radio_stations.json"
    with open(file_path, "w") as outfile:
        json.dump(radio_urls, outfile, indent=4)

    print("Radio stations are reset ...")


def cli_radio_check():
    """
    CLI facing function to check which radio stations are online from the file.

    :return:
    """
    radio_stations = read_stations()
    valid_media_extensions = ["audio/mpeg", "audio/aac", "application/ogg"]
    for station_name, station_url in radio_stations.items():
        resp_content_type = requests.get(station_url, stream=True, timeout=50).headers.get(
            "Content-Type"
        )
        if resp_content_type in valid_media_extensions:
            print(f"[OK!] {station_name} ->>  {station_url}")
        else:
            print(
                f"[FAIL!] {station_name} ->>  {station_url}\nChange or delete this station."
            )


if __name__ == "__main__":
    cli_radio_station_list()



"""
Handles parsing of podcasts
"""
from collections import namedtuple

import click
import feedparser
from prettytable import PrettyTable, ALL
from src.player import player
import requests
from requests import ConnectionError


podcast_urls = {
    "stackoverflow": "https://feeds.simplecast.com/XA_851k3",
    "real_python": "https://realpython.com/podcasts/rpp/feed",
    "python_bytes": "https://pythonbytes.fm/episodes/rss_full_history",
    "talk_python": "https://talkpython.fm/episodes/rss_full_history",
    "data_engineering": "https://www.dataengineeringpodcast.com/feed/mp3/",
    "test_n_code": "https://feeds.fireside.fm/testandcode/rss",
    "profitable_python": "https://anchor.fm/s/c8df638/podcast/rss",
    "teaching_python": "https://feeds.fireside.fm/teachingpython/rss",
    "python_podcast": "https://www.pythonpodcast.com/feed/mp3/",
}


def read_podcasts():
    # MAYBE: maybe make podcast in file?
    return podcast_urls


def title_modifier(podcast_title, max_line_word_count=5):
    """
    Utility function to add new line character to the podcast title if it's too long.
    Increases readability in terminal.

    :param podcast_title: title of the podcast.
    :param max_line_word_count: maximum number of words allowed on single line before inserting new line character.
    :return: modified string with new line characters.
    """
    modified_title_list = podcast_title.split()
    total_word_count = len(modified_title_list)

    for word_index in range(max_line_word_count, total_word_count, max_line_word_count):
        modified_title_list[word_index - 1] = modified_title_list[word_index - 1] + "\n"

    return " ".join(modified_title_list)


def print_via_pager(data, data_len):
    """
    A utility pager function, because click.echo_via_pager() is broken for pretty_tables.
    Prints until data_len is 0 and then quits.

    :param data_len: length of data.
    :param data: data to be printed.
    :return:
    """
    # Defines the starting step for each paging print
    paging_start_step = 0

    # Defines how many elements are printed at a time.
    paging_step = 50
    pager_quit_control = None
    while pager_quit_control != "q" and data_len > 0:
        data.start = paging_start_step
        data.end = paging_start_step + paging_step
        click.echo(data)
        paging_start_step += paging_step
        data_len -= paging_step
        click.echo("Press ENTER for more, q to quit.")
        pager_quit_control = click.getchar().lower()


def podcast_extractor(podcast_name):
    """
    Returns a list of namedtuple(Episode) with id, episode title, episode number, and stream url.

    :param podcast_name: rss feed link for the podcast.
    :return: [Episode(['id', 'episode_title', 'episode_number', 'date', 'stream_url'])]
    """
    podcast_link = read_podcasts().get(podcast_name)
    if podcast_link:
        try:
            requests.get(podcast_link, timeout=50)
        except ConnectionError:
            print("Either you are offline or the site is ...")
        except requests.exceptions.Timeout:
            print("Request Timeout when accessing podcast link ...")
        except requests.exceptions.RequestException as e:
            print(e)
            print("Something went wrong, when accessing podcast ...")
        else:
            all_episodes = []
            Episode = namedtuple(
                "Episode",
                ["id", "episode_title", "episode_number", "date", "stream_url"],
            )

            rss_feed = feedparser.parse(podcast_link)
            episode_entries = rss_feed.entries
            for index, episode_item in enumerate(episode_entries):
                all_episodes.append(
                    Episode(
                        id=index,
                        episode_title=title_modifier(episode_item.get("title", "NULL")),
                        episode_number=episode_item.get("itunes_episode", "NULL"),
                        date=" ".join(
                            episode_item.get("published", "NULL").split()[:-2]
                        ),
                        stream_url=episode_item.links[-1].get("href", "NULL"),
                    )
                )

            return all_episodes
    else:
        print("Incorrect podcast name, try again.")


def cli_print_episodes(podcast_name):
    """
    CLI facing function to pretty print the episode data.

    :param podcast_name:  podcast name from podcast dict.
    :return:
    """
    click.echo("Wait for a few seconds ...")
    episode_list = podcast_extractor(podcast_name)
    if episode_list:
        pretty_table = PrettyTable()
        pretty_table.field_names = ["id", "Title", "Episode", "Date", "Url"]
        pretty_table.add_rows(episode_list)
        pretty_table.hrules = ALL

        pretty_table.fields = ["id", "Title", "Episode"]
        episodes_len = len(episode_list)
        print_via_pager(pretty_table, episodes_len)


def cli_podcast_list():
    """
    CLI facing function to pretty print all available podcast name.

    :return:
    """
    pretty_table = PrettyTable()
    pretty_table.hrules = ALL
    pretty_table.field_names = ["id", "Podcast Name"]
    podcast_list = read_podcasts()
    podcast_list = [
        (index, podcast) for index, podcast in enumerate(podcast_list, start=0)
    ]
    pretty_table.add_rows(podcast_list)
    click.echo(pretty_table)


def cli_podcast_play(podcast_name, episode_id):
    """
    CLI facing function to play the podcast episode, when passed podcast name and episode id, (NOT episode number).

    :param podcast_name: name of the podcast.
    :param episode_id: id(index) of episode.
    :return:
    """
    episode = None
    print("Wait for a few seconds ...\n")
    episode_list = podcast_extractor(podcast_name)
    if episode_list:
        try:
            episode = episode_list[episode_id]
        except IndexError:
            print("Episode id is incorrect")
        except Exception as e:
            print(e)
            print("Something went wrong, when getting podcast episode data to play.")
        else:
            # episode title without new line characters.
            cleaned_episode_title = episode.episode_title.replace("\n", "")
            print(
                f"Episode:: {episode.episode_number} ->> Title:: {cleaned_episode_title}\nDate:: {episode.date}"
            )
            player(episode.stream_url)


if __name__ == "__main__":
    cli_podcast_list()



from setuptools import setup, find_namespace_packages

with open("README.md", 'r', encoding="utf-8") as file:
    long_description = file.read()

setup(
    name='dev-radio',
    version='1.0.10',
    packages=find_namespace_packages(),
    include_package_data=True,
    description="A simple cmdline radio to listen to python podcasts and some music.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Legion-God/dev-radio',
    author="Shubhendra Rajkarne",
    author_email='srajkarne11@gmail.com',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Multimedia :: Sound/Audio"
    ],
    install_requires=[
        'requests',
        'feedparser',
        'click',
        'prettytable',
        'python-vlc'
    ],
    extra_require={
        'dev': ['pytest']
    },
    entry_points='''
        [console_scripts]
        dradio=src.main:dradio
    ''',
    python_requires=">=3.6",
)
